/*  @(#)fileio.c	1.5  97/12/23  */

/*
 * Copyright (c) Communications Research Centre, Government of Canada 1997. 
 * All rights reserved.
 * 
 * License is granted to copy and use this software, for research and
 * evaluation purposes, provided that the Communications Research Centre
 * is acknowledged in all documentation pertaining to any such copy or
 * use. The Communications Research Centre grants no other licenses
 * expressed or implied. Title, ownership rights, and intellectual
 * property rights in the software shall remain with the Communications
 * Research Centre.  The Centre's name and the Government of Canada
 * should not be used in any advertising without written permission.
 * 
 * THE COMMUNICATIONS RESEARCH CENTRE MAKES NO REPRESENTATIONS CONCERNING
 * EITHER THE MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PARTICULAR PURPOSE.  The software is provided
 * "as is" without express or implied warranty of any kind.
 * 
 * These notices must be retained in any copies of any part of this
 * software.
 */


#include "mpoll.h"

char *homedir;

/* -------------------------------------------------------------- */
/* get the user's home directory */
gethomedir()
   {
   char *end;
   int len;
   
   homedir=(char *)getenv("HOME");
   if (! homedir)
      {
      homedir = strdup(".");
      return(0);
      }
   /* remove any trailing slash */
   len = strlen(homedir);
   end = homedir + len - 1;
   if (*end == '/' || *end == '\\')
      {
      *end = '\0';
      }
   return(0);
   }


/* -------------------------------------------------------------- */
/* read the cache file */
readdata()
   {
   FILE *df;
   char cachename[256];
   char *cachefile;
   char line[1024], *fileend, *strstrt();
   char *buff = line;
   int ll = 1024;
   char tmpl[1024];
   char *tmpp = tmpl;
   
   cachefile = strdup(CACHEFILE);
   sprintf(cachename, "%s/%s", homedir, cachefile);

   if ((df = fopen(cachename, "r")) == NULL)
      {
      return(0);
      }
   
   fileend = fgets(buff, ll, df);
   while (fileend)
      {
      linetostr(buff); /* remove trailing \n */
      
      /* a TYPE line starts out a new topic or response */
      if ((tmpp = strstrt("TYPE:question", buff)) != NULL)
         {
         readquestion(df);
         }
      else if ((tmpp = strstrt("TYPE:response", buff)) != NULL)
         {
         readresponse(df);
         }
      fileend = fgets(buff, ll, df);
      }
   }


/* -------------------------------------------------------------- */
/* read the question info into the topics structure */
readquestion(FILE *df)
   {
   char line[1024], *fileend, *strstrt();
   char *buff = line;
   int ll = 1024;
   char tmpl[1024];
   char *tmpp = tmpl;
   struct topic tmptopic;
   int responsenum; /* number of responses */
   
   responsenum = 0;
   memset(&tmptopic, 0, sizeof(struct topic));
   
   fileend = fgets(buff, ll, df);
   
   while (fileend)
      {
      linetostr(buff); /* remove trailing \n */
      
      /* look for comments and ignore them */
      if ((tmpp = strstrt("#", buff)) != NULL)
         {
         fileend = fgets(buff, ll, df);
         continue;
         }
         
      else if ((tmpp = strstrt("QID:", buff)) != NULL)
         {
         tmptopic.qid = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("THEME:", buff)) != NULL)
         {
         tmptopic.theme = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("VERSION:", buff)) != NULL)
         {
         tmptopic.version = atol(tmpp);
         }
         
      else if ((tmpp = strstrt("EMAIL:", buff)) != NULL)
         {
         tmptopic.email = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("SUMMARY:", buff)) != NULL)
         {
         tmptopic.summary = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("QUESTION:", buff)) != NULL)
         {
         tmptopic.question = strdup(tmpp);
         }
      
      else if ((tmpp = strstrt("OPTION:", buff)) != NULL)
         {
         tmptopic.option = strdup(tmpp);
         }
      
      else if ((tmpp = strstrt("DECAY:", buff)) != NULL)
         {
         tmptopic.decay_time = atoi(tmpp);
         }
      
      else if ((tmpp = strstrt("REMIND:", buff)) != NULL)
         {
         tmptopic.remind_time = atoi(tmpp);
         }
      
      else if ((tmpp = strstrt("TIME:", buff)) != NULL)
         {
         tmptopic.time = atol(tmpp);
         }
      
      /* handle the multiple responses that are possible */
      else if ((tmpp = strstrt("RESPONSE:", buff)) != NULL)
         {
         responsenum++;
         tmptopic.response[responsenum] = strdup(tmpp);
         }
         
      /* if blank line, stop reading and process data */
      else if (strcmp("", buff) == 0)
         {
         if (! tmptopic.theme)
            {
            warning("No theme found in cache file");
            return(0);
            }
         if (strcmp(tmptopic.theme, theme) != 0)
         /* not the current theme */
            {
            return(0);
            }
         topicnum++;
         numtopics++;
         memcpy(&topiclist[topicnum], &tmptopic, sizeof(struct topic));
         return(0);
         }
         
      else
         {
         printf("WARNING: could not parse this line in data file:\n   %s\n", buff);
         }
         
      fileend = fgets(buff, ll, df);
      
      }  /* file reading loop */
   
   }
   
/* -------------------------------------------------------------- */
/* read the response info into the topics structure */
readresponse(FILE *df)
   {
   char line[1024], *fileend, *strstrt();
   char *buff = line;
   int ll = 1024;
   char tmpl[1024];
   char *tmpp = tmpl;
   struct response tmpresponse;
   int responsenum; /* number of responses */
   
   responsenum = 0;
   memset(&tmpresponse, 0, sizeof(struct response));
   
   fileend = fgets(buff, ll, df);
   
   while (fileend)
      {
      linetostr(buff); /* remove trailing \n */
      
      /* look for comments and ignore them */
      if ((tmpp = strstrt("#", buff)) != NULL)
         {
         fileend = fgets(buff, ll, df);
         continue;
         }
         
      else if ((tmpp = strstrt("ID:", buff)) != NULL)
         {
         tmpresponse.id = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("THEME:", buff)) != NULL)
         {
         tmpresponse.theme = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("VERSION:", buff)) != NULL)
         {
         tmpresponse.version = atol(tmpp);
         }
         
      else if ((tmpp = strstrt("USER:", buff)) != NULL)
         {
         tmpresponse.user = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("ADDRESS:", buff)) != NULL)
         {
         tmpresponse.address = strdup(tmpp);
         }
         
      else if ((tmpp = strstrt("TIME:", buff)) != NULL)
         {
         tmpresponse.time = atol(tmpp);
         }
      
      /* handle the multiple responses that are possible */
      else if ((tmpp = strstrt("RESPONSE:", buff)) != NULL)
         {
         responsenum++;
         tmpresponse.response[responsenum] = strdup(tmpp);
         }
         
      /* if blank line, stop reading and process data */
      else if (strcmp("", buff) == 0)
         {
         if (! tmpresponse.theme)
            {
            warning("No theme found in cache file");
            return(0);
            }
         if (strcmp(tmpresponse.theme, theme) != 0)
         /* not the current theme */
            {
            return(0);
            }
         nummyresponses++;
         memcpy(&myresponse[nummyresponses], &tmpresponse, sizeof(struct response));
         numresponses++;
         memcpy(&responselist[numresponses], &tmpresponse, sizeof(struct response));
         return(0);
         }
         
      else
         {
         printf("WARNING: could not parse this line in data file:\n   %s\n", buff);
         }
         
      fileend = fgets(buff, ll, df);
      
      }  /* file reading loop */
   
   }

/* -------------------------------------------------------------- */
/* write the data structures to a cache file*/

writedata()
   {
   int i, j;
   FILE *wf;
   char *cachefile;
   char cachename[256];
   unsigned long remoteip;
   char *remoteadd = NULL;
   
   cachefile = strdup(CACHEFILE);
   sprintf(cachename, "%s/%s", homedir, cachefile);

   sprintf(messbuf, "MPoll: writing cache file %s...\n", cachename);
   logger(messbuf);
   if ((wf = fopen(cachename, "w")) == NULL)
      {
      perror("problem opening data file for writing");
      sprintf(messbuf, "Tried to open file: %s\n", cachename);
      perror(messbuf);
      return(1);
      }
   
   /* store my questions */
   for (i = 1; i <= numtopics; i++)
      {
      /* check that this is one of my questions */
      remoteadd = getword(topiclist[i].qid, 2);
      remoteip = inet_addr(remoteadd);
      if (remoteip != hostaddr) /* if not from local host */
         {
         continue;
         }
      
      fprintf(wf, "TYPE:question\n");
      fprintf(wf, "QID:%s\n", topiclist[i].qid);
      fprintf(wf, "THEME:%s\n", topiclist[i].theme);
      fprintf(wf, "VERSION:%lu\n", topiclist[i].version);
      fprintf(wf, "EMAIL:%s\n", topiclist[i].email);
      fprintf(wf, "SUMMARY:%s\n", topiclist[i].summary);
      fprintf(wf, "QUESTION:%s\n", topiclist[i].question);
      for (j = 1; j <= MAX_RESPONSES; j++)
         {
         if (topiclist[i].response[j] != NULL)
            {
            fprintf(wf, "RESPONSE:%s\n", topiclist[i].response[j]);
            }
         }
      fprintf(wf, "OPTION:%s\n", topiclist[i].option);
      fprintf(wf, "DECAY:%d\n", topiclist[i].decay_time);
      fprintf(wf, "REMIND:%d\n", topiclist[i].remind_time);
      fprintf(wf, "TIME:%lu\n", topiclist[i].time);
      fprintf(wf, "\n"); /* extra blank line after group */
      }
      
   /* store my responses */
   for (i = 1; i <= nummyresponses; i++)
      {
      fprintf(wf, "TYPE:response\n");
      fprintf(wf, "ID:%s\n", myresponse[i].id);
      fprintf(wf, "THEME:%s\n", myresponse[i].theme);
      fprintf(wf, "USER:%s\n", myresponse[i].user);
      fprintf(wf, "ADDRESS:%s\n", myresponse[i].address);
      fprintf(wf, "VERSION:%lu\n", myresponse[i].version);
      for (j = 1; j <= MAX_RESPONSES; j++)
         {
         if (myresponse[i].response[j] != NULL)
            {
            fprintf(wf, "RESPONSE:%s\n", myresponse[i].response[j]);
            }
         }
      fprintf(wf, "TIME:%lu\n", myresponse[i].time);
      fprintf(wf, "\n"); /* extra blank line after group */
      }
   
   fclose(wf);
   free(cachefile);
   free(remoteadd);
   }
   
   
/* -------------------------------------------------------------- */
/* write the results to a text file*/

writeresults(char *resfile)
   {
   FILE *rf;
   char resname[256];
   time_t now, tmptime, ntpnow, tdiff;
   int i, j;
   int *cdata;
   int *count_data();
   
   sprintf(resname, "%s/%s", homedir, resfile);
   
   if ((rf = fopen(resname, "w")) == NULL)
      {
      perror("problem opening results file for writing");
      return(0);
      }
   
   fprintf(rf, "# Results of MPoll Session\n");
   fprintf(rf, "# Theme: %s\n", theme);
   time(&now);
   fprintf(rf, "# Date: %s", ctime(&now));
   fprintf(rf, "\n");
   
   for (i = 1; i <= numtopics; i++)
      {
      tmptime = ntptounix(topiclist[i].time);
      fprintf(rf, "Created by: %s\n", topiclist[i].email);
      fprintf(rf, "Created on: %s", ctime(&tmptime));
      fprintf(rf, "Question Type: %s\n", topiclist[i].option);
      fprintf(rf, "Response Lifetime: %d\n", topiclist[i].decay_time);
      fprintf(rf, "Topic: %s\n", topiclist[i].summary);
      fprintf(rf, "Question: %s\n", topiclist[i].question);
      
      if (strcmp(topiclist[i].option, "text") == 0)
         {
         for (j = 1; j <= numresponses; j++)
            {
            /* match the ID */
            if (strcmp(topiclist[i].qid, responselist[j].id) != 0)
               {
               continue;
               }

            /* check for decay */
            if (topiclist[i].decay_time != 0)
               {
               time(&now);
               ntpnow = unixtontp(now);
               tdiff = ntpnow - responselist[j].time;

               /* life time expressed in minutes so multiply by 60 */
               if (tdiff > (topiclist[i].decay_time * DECAY_FACTOR))
                  {
                  continue;
                  }
               }
            /* if we got to here we have a good response */
            fprintf(rf, "Response: %s\n", responselist[j].response[1]);
            }
         fprintf(rf, "\n"); /* extra blank line after group */
         continue;
         }

      
      /* now the multichoice questions */   
      cdata = count_data(i);
      
      /* find out how many responses for this data set */
      for (j = 1; j <= MAX_RESPONSES; j++)
         {
         if (topiclist[i].response[j] != NULL)
            {
            fprintf(rf, "Response: %s: %d\n", topiclist[i].response[j], cdata[j]);
            }
         }	
      fprintf(rf, "\n"); /* extra blank line after group */
      } /* end of topic loop */
   fclose(rf);
   return(1);
   }
 
