/*  @(#)mpoll.h	1.5  97/12/23  */

/*
 * Copyright (c) Communications Research Centre, Government of Canada 1997. 
 * All rights reserved.
 * 
 * License is granted to copy and use this software, for research and
 * evaluation purposes, provided that the Communications Research Centre
 * is acknowledged in all documentation pertaining to any such copy or
 * use. The Communications Research Centre grants no other licenses
 * expressed or implied. Title, ownership rights, and intellectual
 * property rights in the software shall remain with the Communications
 * Research Centre.  The Centre's name and the Government of Canada
 * should not be used in any advertising without written permission.
 * 
 * THE COMMUNICATIONS RESEARCH CENTRE MAKES NO REPRESENTATIONS CONCERNING
 * EITHER THE MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PARTICULAR PURPOSE.  The software is provided
 * "as is" without express or implied warranty of any kind.
 * 
 * These notices must be retained in any copies of any part of this
 * software.
 */
 
#if UNIX
#undef WIN32
#endif
 
#ifdef WIN32
#include <winsock2.h>
#include <time.h>
#endif

#include <tcl.h>
#include <tk.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#ifndef WIN32
#include <sys/time.h>
#include <pwd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

#ifdef WIN32
/* stuff that should be in winsock2.h but is not */

#define IP_OPTIONS          1           /* set/get IP per-packet options    */
#define IP_MULTICAST_IF     2           /* set/get IP multicast interface   */
#define IP_MULTICAST_TTL    3           /* set/get IP multicast timetolive  */
#define IP_MULTICAST_LOOP   4           /* set/get IP multicast loopback    */
#define IP_ADD_MEMBERSHIP   5           /* add  an IP group membership      */
#define IP_DROP_MEMBERSHIP  6           /* drop an IP group membership      */
#define IP_TTL              7           /* set/get IP Time To Live          */
#define IP_TOS              8           /* set/get IP Type Of Service       */
#define IP_DONTFRAGMENT     9           /* set/get IP Don't Fragment flag   */

struct ip_mreq {
        struct in_addr  imr_multiaddr;  /* IP multicast address of group */
        struct in_addr  imr_interface;  /* local IP address of interface */
};
#endif

/* RAND_MAX is not set for sunos platform */
/* should be set to (2**31) - 1 */
#ifndef RAND_MAX
#define RAND_MAX 2147483647
#endif


/* change the following when shipping */
/* ------------------------------------- */
#define DEBUG

/* set this to 1 for testing, and 0 for production */
#define USEDEFAULTS 0

/* ------------------------------------- */


/* the fonts used in the Tk interface */
#ifdef WIN32
#define DEFAULT_FONT "-*-helvetica-medium-r-normal-*-8-*-iso8859-1"
#define SMALL_FONT "-*-helvetica-medium-r-normal-*-6-*-iso8859-1"
#else
#define DEFAULT_FONT "-*-helvetica-bold-r-normal-*-12-*-iso8859-1"
#define SMALL_FONT "-*-helvetica-medium-r-normal-*-10-*-iso8859-1"
#endif

/* maximum number of topics */
#define MAX_TOPICS 100

/* how many responses per topic */
#define MAX_RESPONSES 10

/* define how many simultaneous members we can support */
#define MAX_MEMBERS 1000

/* how big can the question text be */
#define QUESTION_SIZE 1024

/* some buffer sizes */
#define BUFFER_SIZE 2048
#define TMPSTRLEN 1024

/* how frequently should we update the results & members display (secs) */
/* probably should be about every 5 seconds */
#define UPDATE_TIME 5

/*
 * retransmit intervals in secs for different packet types
 * actual will be somewhere between MIN and 2 * MIN 
 *  times the number of questions or responses in the session
 */

/* questions should be quickly e.g. 10-20 */
#define MIN_INTERVAL_QUESTION 10

/* question deletes can be quite long e.g. 20-60 */
#define MIN_INTERVAL_QUESTION_DELETE 20

/* responses can be very quickly e.g. 5-10 */
#define MIN_INTERVAL_RESPONSE 5

/* member packets can be very quickly e.g. 10-20 */
#define MIN_INTERVAL_MEMBER 10

/* multiplier for calculating expiries */
/* normally should be 60 (for minutes) but can reduce for debugging */
#define DECAY_FACTOR 60

/* default multicast address, port, and TTL */
#define DEFAULT_ADDRESS "224.2.2.200"
#define DEFAULT_PORT 6666
#define DEFAULT_TTL 15

/* variables shared between C and TCL/Tk */
extern char *theme;
extern char *version;
extern char *current_summary;
extern char *current_question;
extern char *current_author;
extern char *current_option;
extern char *current_time;
extern int current_decay;

/* the fonts we will be using */
extern char *default_font;
extern char *small_font;

/* information about the local host and user */
extern char hostname[];
extern char hostip[];
extern unsigned long hostaddr;
extern char username[];
extern char systeminfo[];


/* cache file placed in users home directory */
#define CACHEFILE ".mpoll_cache"

/* variables used to setup sockets */
extern char *address;

#ifdef WIN32
extern u_short port;
extern int ttl;
#else
extern int port;
extern char ttl;
#endif


extern int errno;
extern int recvsock, sendsock;

/* flag to log verbose debugging information */
extern int logging;

/* flag for simulating data for demos */
extern int simulate;

/* the main topics data structure */
/* there is an array of these */
struct topic {
   char *qid;
   char *theme;
   unsigned long version;
   char *email;
   char *summary;
   char *question;
   char *response[MAX_RESPONSES+1];
   char *option;
   int decay_time;
   int remind_time;
   unsigned long time;
   };
extern struct topic topiclist[];
extern struct topic deltopics[]; /* once that have been deleted */

/* the response info structure */
/* there are arrays for all responses received, and 
   responses that I have done */
struct response {
   char *id;
   char *theme;
   unsigned long version;
   char *user;
   char *address;
   char *response[MAX_RESPONSES+1];
   unsigned long time;
   };
extern struct response responselist[];
extern struct response myresponse[];

/* structure for recording the member information */
struct member {
   char *theme;
   char *user;
   char *address;
   char *host;
   char *os;
   char *version;
   unsigned long time;
   };
extern struct member memberlist[];

/* counters for the number of topics and responses */
extern int numtopics, topicnum;
extern int numresponses, responsenum;
extern int nummyresponses, myresponsenum;

/*  keep track of how many topics have been deleted */
extern int numdeleted;

/* counter for number of members */
extern int nummembers;


/* some internal functions that are passed in other functions, so
   have to be defined globally */
extern int hear();
extern int listtopics();

extern char* getword();

/* some scratch space */
extern char messbuf[];

