/* @(#)socket.c	1.5 97/12/23 */


/*
 * Copyright (c) Communications Research Centre, Government of Canada 1997. 
 * All rights reserved.
 * 
 * License is granted to copy and use this software, for research and
 * evaluation purposes, provided that the Communications Research Centre
 * is acknowledged in all documentation pertaining to any such copy or
 * use. The Communications Research Centre grants no other licenses
 * expressed or implied. Title, ownership rights, and intellectual
 * property rights in the software shall remain with the Communications
 * Research Centre.  The Centre's name and the Government of Canada
 * should not be used in any advertising without written permission.
 * 
 * THE COMMUNICATIONS RESEARCH CENTRE MAKES NO REPRESENTATIONS CONCERNING
 * EITHER THE MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF
 * THIS SOFTWARE FOR ANY PARTICULAR PURPOSE.  The software is provided
 * "as is" without express or implied warranty of any kind.
 * 
 * These notices must be retained in any copies of any part of this
 * software.
 */


#include "mpoll.h"

int sendsock, recvsock; /* socket file handles */
struct sockaddr_in sockin; /* address for sockets */
struct sockaddr_in sockout; /* address for sockets */
struct ip_mreq mreq; /* multicast address info */
struct in_addr gadd;
char buf[BUFFER_SIZE]; /* input buffer */
int len;
int one = 1;
struct hostent *fromhp;
char *fromip = NULL;
char *toip = NULL;
char *bptr = NULL;

char packbuff[BUFFER_SIZE];
char packpack[BUFFER_SIZE];

/* ----------------------------------------------------------------- */
/* this is used to define the type of sockets under WIN32 */
#ifdef WIN32
init_win_sockets()
	{
   WORD wVersionRequested;
   WSADATA wsaData;
   int wsockerr;

   /* the new Winsock 2 iinterface has to be started */ 
   wVersionRequested = MAKEWORD( 1, 1 );

   wsockerr = WSAStartup( wVersionRequested, &wsaData );
   if ( wsockerr != 0 ) 
      {
      printf("Something wrong with startup of sock interface\n");
      exit(1);
      }

	}
#endif

/* ----------------------------------------------------------------- */

init_socket()
   {
   
   sprintf(messbuf, "Setting up to use address/port/ttl: %s/%d/%d\n", address, port, ttl);
   logger(messbuf);
   
   /* receive socket */
   if ((recvsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
      {
      perror("receive socket");
      exit(1);
      }
      
   sockin.sin_family = AF_INET;
   sockin.sin_port = htons(port);
   
   #ifdef WIN32
   sockin.sin_addr.s_addr=INADDR_ANY;
   #else
   gadd.s_addr = inet_addr(address);
   mreq.imr_multiaddr = gadd;
   mreq.imr_interface.s_addr = htonl(INADDR_ANY);
   sockin.sin_addr = mreq.imr_multiaddr;
   #endif
   
   
      
   /* make sure more than one process can use this port */
   if (setsockopt(recvsock, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one)) == -1)
      perror("setsockopt: reuseaddr");
   
   if (bind(recvsock, (struct sockaddr *)&sockin, sizeof(sockin)) < 0) 
      {
      perror("bind");
      printf("recvsock = {%d}, size = {%d}\n", recvsock, sizeof(sockin));
      exit(1);
      }
   
   /* now do the multicast addressing for WIN32 */   
   #ifdef WIN32
   gadd.s_addr = inet_addr(address);
   mreq.imr_multiaddr = gadd;
   mreq.imr_interface.s_addr = htonl(INADDR_ANY);
   sockin.sin_addr = mreq.imr_multiaddr;
   #endif
      
   /* joining multicast group */
   if (IN_CLASSD(ntohl(mreq.imr_multiaddr.s_addr))) 
      {
      if (setsockopt(recvsock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&mreq, sizeof(mreq)) < 0) 
         {
         perror("IP_ADD_MEMBERSHIP");
         exit(1);
         }
      }
         
         
   /* transmit socket */
   if ((sendsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
      {
      perror("transmit socket");
      exit(1);
      }
      
   if (setsockopt(sendsock, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one)) == -1)
      perror("setsockopt: reuseaddr on sendsock ");
 
   /* set the TTL on the transmit socket */
   
#ifdef WIN32
   
   /* under WIN32, we seem to have to bind to the sending port in order
      to get the TTL option setting to work */
   
   sockout.sin_family = AF_INET;
   sockout.sin_port = htons(port);
   sockout.sin_addr.s_addr = INADDR_ANY;

   if (bind(sendsock, (struct sockaddr *)&sockout, sizeof(sockout)) < 0) 
      {
      perror("bind");
      printf("sendsock = {%d}, size = {%d}\n", sendsock, sizeof(sockout));
      exit(1);
      }

   if (setsockopt(sendsock, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&ttl, sizeof(ttl)) < 0)
         {
         perror("IP_MULTICAST_TTL");
         exit(1);
         }
         
#else
   
   if (setsockopt(sendsock, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl)) < 0)
         {
         perror("IP_MULTICAST_TTL");
         exit(1);
         }
#endif
    
    }

/* ----------------------------------------------------------------- */
/* handle an incoming packet */
/* gets called each time there is something waiting on the incoming
   socket */
   
hear()
   {      
   fd_set fdset;
   struct timeval tv;
   struct sockaddr_in sfrom;
   int flen;
   
   tv.tv_usec = 0;
   tv.tv_sec = 0; /* set time out for fast polling */
   
   FD_ZERO(&fdset);
   FD_SET(recvsock, &fdset); /* add this socket to the watch list */
   select(recvsock+1, &fdset, 0, 0, &tv); /* watch the sockets */
   
   flen = sizeof(struct sockaddr); /* must have enough size here */
   
   if (FD_ISSET(recvsock, &fdset) != 0) /* if socket is ready */
      {
      memset(buf, 0, sizeof(buf));
      len = recvfrom(recvsock, buf, sizeof(buf), 0, (struct sockaddr *)&sfrom, (int *)&flen);
      if (len < 0)
         {
         perror("receive");
         exit(1);      
         }
      else
         {
         /* found out who is calling */
         fromip = (char *)strdup(inet_ntoa(sfrom.sin_addr));
         
         bptr = &buf[0];
         
         if (logging)
            {
            sprintf (messbuf, "\nFrom host %s I got %d bytes of data\nBuf is {%s}\n", fromip, len, bptr);
            logger(messbuf);
            }
            
         /* pass to filter to determine type of packet */
         parse_packet(bptr);
         ET(update idletasks);
         }
      }
   else /* socket is not ready */
      {
      warning("socket not ready"); 
      fflush(stdout);
      }
   free(fromip);
   }
   
/* ------------------------------------------------------------- */
/* send some data out the transmit socket */

talk(char *data)
   {
   len = sendto(sendsock, data, strlen(data), 0, (struct sockaddr *)&sockin, sizeof(sockin));
   if (len < 0)
      {
      perror("sendto");
      exit(1);      
      }
   else
      {
      toip = (char *)strdup(inet_ntoa(sockin.sin_addr));
      if (logging)
         {
         sprintf (messbuf, "\nTo host %s I sent %d bytes of data\n", toip, len);
         logger(messbuf);
         sprintf(messbuf, "Data was {%s}, length was %d\n", data, strlen(data));
         logger(messbuf);
         }
      }
   free(toip);
   ET(update idletasks);
   }
   
/* ----------------------------------------------------------------- */
/* transmit a question packet */

send_question_packet(int num)
   {
   
   char *pack = NULL;
   char *buff = NULL;
   int j;
   
   buff = packbuff;
   memset(buff, 0, sizeof(packbuff));
   pack = packpack;
   memset(pack, 0, sizeof(packpack));
   
      
   /* get question packet data ready for sending */
   sprintf(buff, "TYPE:question\n");
   strcat(pack, buff);
   
   sprintf(buff, "QID:%s\n", topiclist[num].qid);
   strcat(pack, buff);
   
   sprintf(buff, "THEME:%s\n", topiclist[num].theme);
   strcat(pack, buff);
   
   sprintf(buff, "VERSION:%lu\n", topiclist[num].version);
   strcat(pack, buff);
   
   sprintf(buff, "EMAIL:%s\n", topiclist[num].email);
   strcat(pack, buff);
   
   sprintf(buff, "SUMMARY:%s\n", topiclist[num].summary);
   strcat(pack, buff);
   
   sprintf(buff, "QUESTION:%s\n", topiclist[num].question);
   strcat(pack, buff);
   
   for (j = 1; j <= MAX_RESPONSES; j++)
      {
      if (topiclist[num].response[j] != NULL)
         {
         sprintf(buff, "RESPONSE:%s\n", topiclist[num].response[j]);
         strcat(pack, buff);
         }
      }
      
   sprintf(buff, "OPTION:%s\n", topiclist[num].option);
   strcat(pack, buff);
   
   sprintf(buff, "DECAY:%d\n", topiclist[num].decay_time);
   strcat(pack, buff);
   
   sprintf(buff, "REMIND:%d\n", topiclist[num].remind_time);
   strcat(pack, buff);
   
   sprintf(buff, "TIME:%lu\n", topiclist[num].time);
   strcat(pack, buff);
   
   /* now send the packet */
   talk(pack); 
   sprintf(messbuf, "OUT: question %d", num);
   traffic(messbuf);
   
   }
   
   
/* ----------------------------------------------------------------- */
/* queue all my questions found in the cache file */

queue_all_questions()
   {
   int i;
   unsigned long remoteip;
   char *remoteadd = NULL;
   Tcl_TimerToken timer_token;
   int interval, adjust;
   int littleinterval;
   
   for (i=1; i <= numtopics; i++)
      {
      /* check that topic is one I created */
      /* convert remote address to UNIX IP address */
      remoteadd = getword(topiclist[i].qid, 2);
      remoteip = inet_addr(remoteadd);
      free(remoteadd);
    
      if (remoteip != hostaddr) /* if not from local host */
         {
         continue;
         }
      /* send it */
      littleinterval = i * 500; /* interval is msecs */
      timer_token =  Tcl_CreateTimerHandler(littleinterval, 
        (Tk_TimerProc*)send_question_packet, (ClientData)i);
      
      /* send_question_packet(i); */
      }
      
   /* now arrange for them to be sent again */

   /* pick and interval based on the MIN_INTERVAL */
   adjust = rand() / (RAND_MAX / MIN_INTERVAL_QUESTION + 1);
   
   /* scale for number of topics  */
   adjust = adjust + numtopics; 
   
   /* retransmit interval is in msecs so... */
   interval = (MIN_INTERVAL_QUESTION + adjust) * 1000; 
   
   /* printf("Question interval is %d\n", interval / 1000); */

   timer_token =  Tcl_CreateTimerHandler(interval, 
     (Tk_TimerProc*)queue_all_questions, (ClientData)NULL);
   
   return TCL_OK;
   
   }
   
/* -------------------------------------------------------------- */
/* send a question deletion message */

send_question_delete(int num)
   {
   char *pack = NULL;
   char *buff = NULL;
   
   buff = packbuff;
   memset(buff, 0, sizeof(packbuff));
   pack = packpack;
   memset(pack, 0, sizeof(packpack));

   sprintf(buff, "TYPE:delete_question\n");
   strcat(pack, buff);
   
   sprintf(buff, "ID:%s\n", deltopics[num].qid);
   strcat(pack, buff);
   
   sprintf(buff, "THEME:%s\n", deltopics[num].theme);
   strcat(pack, buff);
   
   sprintf(buff, "ADDRESS:%s\n", hostip);
   strcat(pack, buff);
   
   sprintf(buff, "USER:%s\n", username);
   strcat(pack, buff);

   /* now send the packet */
   talk(pack); 
   sprintf(messbuf, "OUT: delete question %d", num);
   traffic(messbuf);
   

   }
   
/* ----------------------------------------------------------------- */
/* queue up a question delete request to be multicast repeatedly */

queue_question_delete(int num)
   {
   Tcl_TimerToken timer_token;
   int interval, adjust;

   send_question_delete(num); /* send the data once*/

   /* now arrange for it to be sent again */

   /* pick and interval between 5 and 10 seconds */
   adjust = rand() / (RAND_MAX / MIN_INTERVAL_QUESTION_DELETE + 1);
   
   /* scale for number of topics */
   /* adjust = adjust * numtopics; */
   
   interval = MIN_INTERVAL_QUESTION_DELETE + adjust; 
   
   /* retransmit interval is in msecs so... */
   interval = interval * 1000; 

   timer_token =  Tcl_CreateTimerHandler(interval, 
     (Tk_TimerProc*)queue_question_delete, (ClientData)num);

   return TCL_OK;
   
   }
   
   
   
/* -------------------------------------------------------------- */
/* prepare a response packet for sending */

send_response_packet(int num)
   {
   char *pack = NULL;
   char *buff = NULL;
   int j;
   
   buff = packbuff;
   memset(buff, 0, sizeof(packbuff));
   pack = packpack;
   memset(pack, 0, sizeof(packpack));
   
   sprintf(buff, "TYPE:response\n");
   strcat(pack, buff);
   
   sprintf(buff, "ID:%s\n", myresponse[num].id);
   strcat(pack, buff);
   
   sprintf(buff, "THEME:%s\n", myresponse[num].theme);
   strcat(pack, buff);
   
   sprintf(buff, "USER:%s\n", myresponse[num].user);
   strcat(pack, buff);
   
   sprintf(buff, "ADDRESS:%s\n", myresponse[num].address);
   strcat(pack, buff);
   
   sprintf(buff, "VERSION:%lu\n", myresponse[num].version);
   strcat(pack, buff);
   
   /* must handle all the responses here */
   for (j = 1; j <= MAX_RESPONSES; j++)
      {
      if (myresponse[num].response[j] != NULL)
         {
         sprintf(buff, "RESPONSE:%s\n", myresponse[num].response[j]);
         strcat(pack, buff);
         }
      }
   
   sprintf(buff, "TIME:%lu\n", myresponse[num].time);
   strcat(pack, buff);
   
   /* now send the packet */
   talk(pack); 
   sprintf(messbuf, "OUT: response %d", num);
   traffic(messbuf);
   
   }
   
   
/* -------------------------------------------------------------- */
/* arrange for response packet to be repeated  */

queue_all_responses()
   {
   Tcl_TimerToken timer_token;
   int interval, adjust;
   int t, i;
   time_t now, ntpnow;
   long int tdiff;
   int littleinterval;
   
   for (i = 1; i <= nummyresponses; i++)
      {
      /* check that this response had not decayed to expiry */
      /* get the correct item in the topic structure */
      for (t = 1; t <= numtopics; t++)
         {
         if (strcmp(myresponse[i].id, topiclist[t].qid) == 0)
            {
            /* t will be the appropriate topic */
            break;
            }
         }
      if (topiclist[t].decay_time != 0)
         {
         time(&now);
         ntpnow = unixtontp(now);
         tdiff = ntpnow - myresponse[i].time;

         /* life time expressed in minutes so multiply by 60 */
         if (tdiff > (topiclist[t].decay_time * DECAY_FACTOR))
            {
            continue;
            }
         }
      littleinterval = i * 500; /* interval is msecs */
      timer_token =  Tcl_CreateTimerHandler(littleinterval, 
        (Tk_TimerProc*)send_response_packet, (ClientData)i);
        
      /* send_response_packet(i);  */
      }
  
   /* now arrange for it to be sent again */

   /* pick and interval between 5 and 10 seconds */
   /* should scale for number of topics/responses */
   adjust = rand() / (RAND_MAX / MIN_INTERVAL_RESPONSE + 1);
   
   /* scale by the total number of responses ??? */
   /* interval = (MIN_INTERVAL_RESPONSE + adjust) * (numresponses + 1); */ 
   interval = MIN_INTERVAL_RESPONSE + adjust + nummyresponses; 
   
   interval = interval * 1000; 
   /* retransmit interval is in msecs */

   timer_token =  Tcl_CreateTimerHandler(interval, 
     (Tk_TimerProc*)queue_all_responses, (ClientData)NULL);

   return TCL_OK;
   
   }  

/* -------------------------------------------------------------- */
/* send membership packet   */

send_member_packet()
   {
   char *pack = NULL;
   char *buff = NULL;
   
   buff = packbuff;
   memset(buff, 0, sizeof(packbuff));
   pack = packpack;
   memset(pack, 0, sizeof(packpack));
   
   sprintf(buff, "TYPE:member\n");
   strcat(pack, buff);
   
   sprintf(buff, "THEME:%s\n", theme);
   strcat(pack, buff);
   
   sprintf(buff, "USER:%s\n", username);
   strcat(pack, buff);
   
   sprintf(buff, "ADDRESS:%s\n", hostip);
   strcat(pack, buff);
   
   sprintf(buff, "OS:%s\n", systeminfo);
   strcat(pack, buff);
   
   sprintf(buff, "VERSION:%s\n", version);
   strcat(pack, buff);
   
   /* now send the packet */
   talk(pack); 
   sprintf(messbuf, "OUT: membership");
   traffic(messbuf);
   
   }
   
   
   
/* -------------------------------------------------------------- */
/* arrange for membership packet to be repeated  */
queue_member()
   {
   Tcl_TimerToken timer_token;
   int interval, adjust;
   
   send_member_packet(); /* send the data once */
   
   /* now arrange for it to be sent again */

   /* pick and interval between 5 and 10 seconds */
   /* should scale for number of topics/responses */
   adjust = rand() / (RAND_MAX / MIN_INTERVAL_MEMBER + 1);
   
   /* scale by the total number of responses */
   /* ASP - should this be scaled by the number of members? */
   interval = (MIN_INTERVAL_MEMBER + adjust); 
   
   interval = interval * 1000; 
   /* retransmit interval is in msecs */

   timer_token =  Tcl_CreateTimerHandler(interval, 
     (Tk_TimerProc*)queue_member, (ClientData)NULL);

   return TCL_OK;
   
   }  
